Re: [U-Boot] [PATCH v2 11/18] efi_loader: make efi_disk_create_partitions a global symbol

2018-01-18 Thread Alexander Graf


On 17.01.18 20:16, Heinrich Schuchardt wrote:
> Up to now we have been using efi_disk_create_partitions() to create
> partions for block device that existed before starting an EFI

partitions

devices

> application.
> 
> We need to to call it for for block devices created by EFI

s/to//
s/for//


Alex
___
U-Boot mailing list
U-Boot@lists.denx.de
https://lists.denx.de/listinfo/u-boot


[U-Boot] [PATCH v2 11/18] efi_loader: make efi_disk_create_partitions a global symbol

2018-01-17 Thread Heinrich Schuchardt
Up to now we have been using efi_disk_create_partitions() to create
partions for block device that existed before starting an EFI
application.

We need to to call it for for block devices created by EFI
applications at run time. The EFI application will define the
handle for the block device and install a device path protocol
on it. We have to use this device path as stem for the partition
device paths.

Signed-off-by: Heinrich Schuchardt 
---
v2
no change
---
 include/efi_loader.h  |  4 +++
 lib/efi_loader/efi_disk.c | 84 +--
 2 files changed, 64 insertions(+), 24 deletions(-)

diff --git a/include/efi_loader.h b/include/efi_loader.h
index 456763e83a..0ba7badb15 100644
--- a/include/efi_loader.h
+++ b/include/efi_loader.h
@@ -174,6 +174,10 @@ extern struct list_head efi_obj_list;
 int efi_console_register(void);
 /* Called by bootefi to make all disk storage accessible as EFI objects */
 int efi_disk_register(void);
+/* Create handles and protocols for the partions of a block device */
+int efi_disk_create_partitions(efi_handle_t parent, struct blk_desc *desc,
+  const char *if_typename, int diskid,
+  const char *pdevname);
 /* Called by bootefi to make GOP (graphical) interface available */
 int efi_gop_register(void);
 /* Called by bootefi to make the network interface available */
diff --git a/lib/efi_loader/efi_disk.c b/lib/efi_loader/efi_disk.c
index cccfc6dac5..92c3f45ca5 100644
--- a/lib/efi_loader/efi_disk.c
+++ b/lib/efi_loader/efi_disk.c
@@ -216,27 +216,31 @@ efi_fs_from_path(struct efi_device_path *full_path)
 }
 
 /*
  * Create a device for a disk
  *
- * @name   not used
+ * @parent parent handle
+ * @dp_parent  parent device path
  * @if_typename interface name for block device
  * @desc   internal block device
  * @dev_index   device index for block device
  * @offset offset into disk for simple partitions
+ * @return disk object
  */
-static void efi_disk_add_dev(const char *name,
-const char *if_typename,
-struct blk_desc *desc,
-int dev_index,
-lbaint_t offset,
-unsigned int part)
+static struct efi_disk_obj *efi_disk_add_dev(
+   efi_handle_t parent,
+   struct efi_device_path *dp_parent,
+   const char *if_typename,
+   struct blk_desc *desc,
+   int dev_index,
+   lbaint_t offset,
+   unsigned int part)
 {
struct efi_disk_obj *diskobj;
efi_status_t ret;
 
/* Don't add empty devices */
if (!desc->lba)
-   return;
+   return NULL;
 
diskobj = calloc(1, sizeof(*diskobj));
if (!diskobj)
@@ -246,7 +250,14 @@ static void efi_disk_add_dev(const char *name,
efi_add_handle(>parent);
 
/* Fill in object data */
-   diskobj->dp = efi_dp_from_part(desc, part);
+   if (part) {
+   struct efi_device_path *node = efi_dp_part_node(desc, part);
+
+   diskobj->dp = efi_dp_append_node(dp_parent, node);
+   efi_free_pool(node);
+   } else {
+   diskobj->dp = efi_dp_from_part(desc, part);
+   }
diskobj->part = part;
ret = efi_add_protocol(diskobj->parent.handle, _block_io_guid,
   >ops);
@@ -280,20 +291,38 @@ static void efi_disk_add_dev(const char *name,
if (part != 0)
diskobj->media.logical_partition = 1;
diskobj->ops.media = >media;
-   return;
+   return diskobj;
 out_of_memory:
printf("ERROR: Out of memory\n");
+   return NULL;
 }
 
-static int efi_disk_create_partitions(struct blk_desc *desc,
- const char *if_typename,
- int diskid,
- const char *pdevname)
+/*
+ * Create handles and protocols for the partions of a block device
+ *
+ * @parent handle of the parent disk
+ * @blk_desc   block device
+ * @if_typenameinterface type
+ * @diskid device number
+ * @pdevname   device name
+ * @return number of partions created
+ */
+int efi_disk_create_partitions(efi_handle_t parent, struct blk_desc *desc,
+  const char *if_typename, int diskid,
+  const char *pdevname)
 {
int disks = 0;
char devname[32] = { 0 }; /* dp->str is u16[32] long */
disk_partition_t info;
int part;
+   struct efi_device_path *dp = NULL;
+   efi_status_t ret;
+   struct efi_handler *handler;
+
+
@@ -216,27 +216,31 @@ efi_fs_from_path(struct efi_device_path *full_path)
 
 /*
- * Create a device for a disk
+ * Create a handle for a partition or disk
  *